@@ -19,10 +19,16 @@
 
 #include <config.h>
 #include "gconf-database.h"
+#ifdef HAVE_DBUS
+#include "gconf-database-dbus.h"
+#endif
 #include "gconf-listeners.h"
 #include "gconf-sources.h"
 #include "gconf-locale.h"
 #include "gconfd.h"
+#ifdef HAVE_DBUS
+#include "gconfd-dbus.h"
+#endif
 #include <string.h>
 #include <unistd.h>
 #include <time.h>
@@ -32,8 +38,7 @@
  * Forward decls
  */
 
-static GConfLocaleList* locale_cache_lookup(const gchar* locale);
-
+#ifdef HAVE_CORBA
 typedef struct _Listener Listener;
 
 struct _Listener {
@@ -95,7 +100,7 @@ impl_ConfigDatabase_lookup_with_locale(PortableServer_Servant servant,
   if (gconfd_check_in_shutdown (ev))
     return gconf_invalid_corba_value ();
   
-  locale_list = locale_cache_lookup(locale);
+  locale_list = gconfd_locale_cache_lookup(locale);
   
   val = gconf_database_query_value(db, key, locale_list->list,
                                    use_schema_default,
@@ -154,7 +159,7 @@ impl_ConfigDatabase_lookup_default_value(PortableServer_Servant servant,
   if (gconfd_check_in_shutdown (ev))
     return gconf_invalid_corba_value ();
   
-  locale_list = locale_cache_lookup(locale);
+  locale_list = gconfd_locale_cache_lookup(locale);
   
   val = gconf_database_query_default_value(db, key,
                                            locale_list->list,
@@ -336,7 +341,7 @@ impl_ConfigDatabase_all_entries(PortableServer_Servant servant,
   if (gconfd_check_in_shutdown (ev))
     return;
   
-  locale_list = locale_cache_lookup(locale);
+  locale_list = gconfd_locale_cache_lookup(locale);
   
   pairs = gconf_database_all_entries(db, dir, locale_list->list, &error);
   
@@ -548,7 +553,7 @@ impl_ConfigDatabase2_lookup_with_schema_name(PortableServer_Servant servant,
   if (gconfd_check_in_shutdown (ev))
     return gconf_invalid_corba_value ();
   
-  locale_list = locale_cache_lookup(locale);
+  locale_list = gconfd_locale_cache_lookup(locale);
 
   s = NULL;
   val = gconf_database_query_value(db, key, locale_list->list,
@@ -614,7 +619,7 @@ impl_ConfigDatabase2_all_entries_with_schema_name(PortableServer_Servant servant
   if (gconfd_check_in_shutdown (ev))
     return;
   
-  locale_list = locale_cache_lookup(locale);
+  locale_list = gconfd_locale_cache_lookup(locale);
   
   pairs = gconf_database_all_entries(db, dir, locale_list->list, &error);
   
@@ -779,19 +784,47 @@ static POA_ConfigDatabase3__epv server3_epv = {
 
 static POA_ConfigDatabase3__vepv poa_server_vepv = { &base_epv, &server_epv, &server2_epv, &server3_epv };
 
+#endif /* HAVE_CORBA */
+
 static void gconf_database_really_sync (GConfDatabase *db);
 static void source_notify_cb           (GConfSource   *source,
 					const gchar   *location,
 					GConfDatabase *db);
 
+void
+gconf_database_set_sources (GConfDatabase *db,
+                            GConfSources  *sources)
+{
+  if (db->sources != NULL)
+    {
+      /* this function should only be used when creating the db with the corba
+       * backend */
+#ifdef HAVE_CORBA
+      gconf_sources_clear_cache(db->sources);
+      gconf_sources_free(db->sources);
+#else
+      g_assert_not_reached ();
+#endif
+    }
+
+  db->sources = sources;
+
+  gconf_sources_set_notify_func (db->sources,
+				 (GConfSourceNotifyFunc) source_notify_cb,
+				 db);
+}
+
 GConfDatabase*
 gconf_database_new (GConfSources  *sources)
 {
   GConfDatabase* db;
+#ifdef HAVE_CORBA
   CORBA_Environment ev;
+#endif
   
   db = g_new0 (GConfDatabase, 1);
 
+#ifdef HAVE_CORBA
   db->servant._private = NULL;
   db->servant.vepv = &poa_server_vepv;
 
@@ -809,14 +842,15 @@ gconf_database_new (GConfSources  *sources)
 
       exit (1);
     }
+#endif
+
+#ifdef HAVE_DBUS
+  gconf_database_dbus_setup (db);
+#endif
 
   db->listeners = gconf_listeners_new();
 
-  db->sources = sources;
-
-  gconf_sources_set_notify_func (db->sources,
-				 (GConfSourceNotifyFunc) source_notify_cb,
-				 db);
+  gconf_database_set_sources(db, sources);
 
   db->last_access = time(NULL);
 
@@ -831,6 +865,7 @@ gconf_database_new (GConfSources  *sources)
 void
 gconf_database_free (GConfDatabase *db)
 {
+#ifdef HAVE_CORBA
   PortableServer_ObjectId *oid;
   CORBA_Environment ev;
 
@@ -853,7 +888,12 @@ gconf_database_free (GConfDatabase *db)
   CORBA_free (oid);
 
   CORBA_exception_free (&ev);
-  
+#endif
+
+#ifdef HAVE_DBUS
+  gconf_database_dbus_teardown (db);
+#endif
+
   if (db->listeners != NULL)
     {
       gboolean need_sync = FALSE;
@@ -886,6 +926,7 @@ gconf_database_free (GConfDatabase *db)
   g_free (db);
 }
   
+#ifdef HAVE_CORBA
 static gboolean
 client_alive_predicate (const gchar* location,
                         guint        cnxn_id,
@@ -915,16 +956,19 @@ client_alive_predicate (const gchar* location,
   
   return result;
 }
+#endif
 
 void
 gconf_database_drop_dead_listeners (GConfDatabase *db)
 {
+#ifdef HAVE_CORBA
   if (db->listeners)
     {
       gconf_listeners_remove_if (db->listeners,
                                  client_alive_predicate,
                                  NULL);
     }
+#endif
 }
 
 static gint
@@ -1025,7 +1069,9 @@ source_notify_cb (GConfSource   *source,
   if (gconf_sources_is_affected (db->sources, source, location))
     {
       GConfValue  *value;
+#ifdef HAVE_CORBA
       ConfigValue *cvalue;
+#endif
       GError      *error;
       gboolean     is_default;
       gboolean     is_writable;
@@ -1052,6 +1098,7 @@ source_notify_cb (GConfSource   *source,
 	  return;
 	}
 
+#ifdef HAVE_CORBA
       cvalue = gconf_corba_value_from_gconf_value (value);
       gconf_database_notify_listeners (db,
 				       NULL,
@@ -1061,10 +1108,22 @@ source_notify_cb (GConfSource   *source,
 				       is_writable,
 				       FALSE);
       CORBA_free (cvalue);
+#endif
+
+#ifdef HAVE_DBUS
+      gconf_database_dbus_notify_listeners (db,
+					    NULL,
+					    location,
+					    value,
+					    is_default,
+					    is_writable,
+					    FALSE);
+#endif
       gconf_value_free (value);
     }
 }
 
+#ifdef HAVE_CORBA
 CORBA_unsigned_long
 gconf_database_readd_listener   (GConfDatabase       *db,
                                  ConfigListener       who,
@@ -1271,6 +1330,7 @@ gconf_database_notify_listeners (GConfDatabase       *db,
       g_free (modified_sources);
     }
 }
+#endif
 
 GConfValue*
 gconf_database_query_value (GConfDatabase  *db,
@@ -1326,7 +1386,9 @@ void
 gconf_database_set   (GConfDatabase      *db,
                       const gchar        *key,
                       GConfValue         *value,
+#ifdef HAVE_CORBA
                       const ConfigValue  *cvalue,
+#endif
                       GError        **err)
 {
   GError *error = NULL;
@@ -1362,6 +1424,7 @@ gconf_database_set   (GConfDatabase      *db,
       /* Can't possibly be the default, since we just set it,
        * and must be writable since setting it succeeded.
        */
+#ifdef HAVE_CORBA
       gconf_database_notify_listeners (db,
 				       modified_sources,
 				       key,
@@ -1369,6 +1432,16 @@ gconf_database_set   (GConfDatabase      *db,
                                        FALSE,
 				       TRUE,
 				       TRUE);
+#endif
+#ifdef HAVE_DBUS
+      gconf_database_dbus_notify_listeners (db,
+					    modified_sources,
+					    key,
+					    value,
+					    FALSE,
+					    TRUE,
+					    TRUE);
+#endif
     }
 }
 
@@ -1378,7 +1451,9 @@ gconf_database_unset (GConfDatabase      *db,
                       const gchar        *locale,
                       GError        **err)
 {
+#ifdef HAVE_CORBA
   ConfigValue* val;
+#endif
   GError* error = NULL;
   GConfSources *modified_sources = NULL;
   
@@ -1428,6 +1503,7 @@ gconf_database_unset (GConfDatabase      *db,
         gconf_log(GCL_ERR, _("Error getting default value for `%s': %s"),
                   key, (*err)->message);
 
+#ifdef HAVE_CORBA
       if (def_value != NULL)
         {
           val = gconf_corba_value_from_gconf_value(def_value);
@@ -1448,6 +1524,21 @@ gconf_database_unset (GConfDatabase      *db,
 				      is_writable,
 				      TRUE);
       CORBA_free(val);
+#endif
+
+#ifdef HAVE_DBUS
+      gconf_database_schedule_sync(db);
+
+      gconf_database_dbus_notify_listeners(db,
+					   modified_sources,
+					   key,
+					   def_value,
+					   TRUE,
+					   is_writable,
+					   TRUE);
+      if (def_value)
+	      gconf_value_free(def_value);
+#endif
     }
 }
 
@@ -1458,7 +1549,9 @@ gconf_database_recursive_unset (GConfDatabase      *db,
                                 GConfUnsetFlags     flags,
                                 GError            **err)
 {
+#ifdef HAVE_CORBA
   ConfigValue* val;
+#endif
   GError* error = NULL;
   GSList *notifies;
   GSList *tmp;
@@ -1519,7 +1612,8 @@ gconf_database_recursive_unset (GConfDatabase      *db,
 	  g_propagate_error (err, error);
 	  error = NULL;
 	}
-      
+
+#ifdef HAVE_CORBA
       if (new_value != NULL)
         {
           val = gconf_corba_value_from_gconf_value (new_value);
@@ -1541,6 +1635,21 @@ gconf_database_recursive_unset (GConfDatabase      *db,
 				       TRUE);
       
       CORBA_free (val);
+#endif
+#ifdef HAVE_DBUS
+      gconf_database_schedule_sync (db);
+      
+      gconf_database_dbus_notify_listeners (db,
+					    notify->modified_sources,
+					    notify->key,
+					    new_value,
+					    is_default,
+					    is_writable,
+					    TRUE);
+      if (new_value)
+	      gconf_value_free (new_value);
+#endif
+
       g_free (notify->key);
       g_free (notify);
       
@@ -1775,6 +1884,7 @@ gconf_database_get_persistent_name (GConfDatabase *db)
   return db->persistent_name;
 }
 
+#ifdef HAVE_CORBA
 struct ForeachData
 {
   GString *str;
@@ -1848,6 +1958,7 @@ gconf_database_log_listeners_to_string (GConfDatabase *db,
 
   g_free (fd.db_name);
 }
+#endif
 
 /*
  * Locale hash
@@ -1855,8 +1966,8 @@ gconf_database_log_listeners_to_string (GConfDatabase *db,
 
 static GConfLocaleCache* locale_cache = NULL;
 
-static GConfLocaleList*
-locale_cache_lookup(const gchar* locale)
+GConfLocaleList*
+gconfd_locale_cache_lookup (const gchar *locale)
 {
   GConfLocaleList* locale_list;
   
@@ -1888,6 +1999,7 @@ gconfd_locale_cache_drop(void)
     }
 }
 
+#ifdef HAVE_CORBA
 /*
  * The listener object
  */
@@ -1919,6 +2031,7 @@ listener_destroy (Listener* l)
   g_free (l->name);
   g_free (l);
 }
+#endif
 
 
 
